@@ -15,6 +15,12 @@ class SemanticEdge(object):
     def __ne__(self, other):
         return not (self == other)
 
+    def __str__(self):
+        if self.comment:
+            return "{self.accession} ! {self.comment}".format(self=self)
+        else:
+            return self.accession
+
     def __repr__(self):
         return "Reference(%r, %r)" % (self.accession, self.comment)
 
@@ -39,7 +45,7 @@ class Relationship(SemanticEdge):
         self.comment = comment
 
     def __str__(self):
-        return "%s: %s ! %s" % (self.predicate, self.accession, self.comment)
+        return "%s %s ! %s" % (self.predicate, self.accession, self.comment)
 
     def __repr__(self):
         return "{self.__class__.__name__}({self.predicate}, {self.accession}, {self.comment})".format(self=self)
